@@ -6,48 +6,38 @@ import multiprocessing as mp
 import threading as mt
 from importlib import reload
 from pathlib import Path
-from PySide2.QtCore import QCoreApplication, QObject, QThread, Qt, QTimer
-from PySide2.QtCore import Signal
+from PySide2.QtCore import QCoreApplication, QObject, QThread, Qt, QRunnable, QThreadPool, QMutex, Signal
+
 
-    
 try:
     from element_types import Record, ProcCMD, GuiCMD
 except ImportError:    
     from Pythonic.element_types import Record, ProcCMD, GuiCMD
 
-"""
-def target_0(instance, record, feed_pipe):
-
-    element = instance(*record)
-    ret = element.execute_ex(record)
-    feed_pipe.send(ret)
-
-    ## return element id und data
-"""
 
 
-class ProcessHandler(QThread):
+class ProcessHandler(QRunnable):
 
-    execComplete  = Signal(object, object, object) # id, data, thread-identifier
-    removeSelf    = Signal(object, object) # id, thread-identifier
+    def __init__(self, element, inputdata, identifier, operator):
+        super(ProcessHandler, self).__init__()
 
-    def __init__(self, element, inputdata, identifier):
-        super().__init__()
         self.element    = element
         self.inputData  = inputdata
         self.identifier = identifier
         self.instance   = None
-        self.element['Config']['Identifier'] = self.identifier
         self.pid        = None
         self.queue      = None 
+        self.operator   = operator
+        self.element['Config']['Identifier'] = self.identifier
 
+        logging.debug('ProcessHandler::__init__() called') 
 
-        self.finished.connect(self.done)
 
     def run(self):
-        #logging.debug('ProcessHandler::run() -id: 0x{:08x}, ident: {:04d}'.format(self.element['Id'], self.identifier))
+        logging.debug('ProcessHandler::run() -id: 0x{:08x}, ident: {:04d}'.format(self.element['Id'], self.identifier))
+
+
         bMP = self.element['Config']['GeneralConfig']['MP']
-        
 
         if bMP:
             self.return_queue = mp.Queue()
@@ -72,17 +62,17 @@ class ProcessHandler(QThread):
                 self.element['Id'], self.identifier, self.element['Filename'], e))
             return
 
-        self.instance = elementCls(self.element['Config'], self.inputData, self.return_queue, self.cmd_queue)
+        self.instance = elementCls(self.element['Id'], self.element['Config'], self.inputData, self.return_queue, self.cmd_queue)
         result = None
 
         
 
         if bMP: ## attach Debugger if flag is set
-            self.p_0 = mp.Process(target=self.instance.execute)
+            self.p_0 = mp.Process(target=self.instance.execute_ex)
             self.p_0.start()
             self.pid = self.p_0.pid
         else:
-            self.t_0 = mt.Thread(target=self.instance.execute)
+            self.t_0 = mt.Thread(target=self.instance.execute_ex)
             self.t_0.start()
 
 
@@ -97,14 +87,16 @@ class ProcessHandler(QThread):
 
         # Check if it is an intemediate result (result.bComplete)
         # or if the execution was stopped by the user (self.element.bStop)
-        
+
+
+
         while not bMP:
 
             try:
                 # First: Check if there is somethin in the Queue
                 result = self.return_queue.get(block=True, timeout=0.2)
                 # Seconds: Forward the result (is present)
-                self.execComplete.emit(self.element['Id'], result, self.identifier)
+                self.operator.operationDone(self.element['Id'], result, self.identifier)
             except queue.Empty:
                 #logging.debug('return_queue empty')
                 pass
@@ -126,8 +118,8 @@ class ProcessHandler(QThread):
 
             try:
                 result = self.return_queue.get(block=True, timeout=0.2)
-                self.execComplete.emit(self.element['Id'], result, self.identifier)
 
+                self.operator.operationDone(self.element['Id'], result, self.identifier)
                 #logging.debug('ProcessHandler::run() - Multiprocessing: execution completed - id: 0x{:08x}, ident: {:04d}, pid: {}'.format(
                 #    self.element['Id'], self.identifier, self.p_0.pid))
             except queue.Empty:
@@ -137,94 +129,34 @@ class ProcessHandler(QThread):
             if not self.p_0.is_alive():
                 break
             
-
-
-        #logging.debug('ProcessHandler::run() - PROCESSING DONE - id: 0x{:08x}, ident: {:04d}'.format(self.element['Id'], self.identifier))
+        self.operator.removeOperatorThread(self.element['Id'], self.identifier)
         
     def stop(self):
         logging.debug('ProcessHandler::stop() - id: 0x{:08x}, ident: {:04d}'.format(self.element['Id'], self.identifier))
-        self.cmd_queue.put(ProcCMD(True))
+        self.cmd_queue.put(ProcCMD(None, True))
 
-    def done(self):
-        #logging.debug('ProcessHandler::done() removing Self - id: 0x{:08x}, ident: {:04d}'.format(self.element['Id'], self.identifier))
-        self.removeSelf.emit(self.element['Id'], self.identifier)
+    def feed(self, data): 
+        logging.debug('ProcessHandler::feed() - id: 0x{:08x}, ident: {:04d}'.format(self.element['Id'], self.identifier))
+        self.cmd_queue.put(ProcCMD(data))
+
+class OperatorStartAll(QRunnable):
     
 
-
-class Operator(QThread):
-
-    currentConfig   = None
-    processHandles  = {}
-    command         = Signal(object) # command
-
-    def __init__(self,):
-        super().__init__()
-
+    def __init__(self, operator):
+        super(OperatorStartAll, self).__init__()
+        self.operator       = operator
+        self.setAutoDelete(False)
+        
     def run(self):
 
-        while True:
-            time.sleep(1)
-
-    def startExec(self, id, config):
-        #logging.debug('Operator::startExec() called - id: 0x{:08x}'.format(id))
-        ## create processor and forward config and start filename
-        self.currentConfig = config
-        # https://stackoverflow.com/questions/34609935/passing-a-function-with-two-arguments-to-filter-in-python
-
-        # return first element which matches the ID
-        startElement = [x for x in config if x['Id'] == id][0]
-
-        self.createProcHandle(startElement)
-
-    def createProcHandle(self, element):    
-        #neue function
-
-        # register elements für den fall das alles gestoppt werden muss
-        inputData = None
-
-        # creating a random identifier
-        identifier = random.randint(0, 9999)
-        runElement = ProcessHandler(element,inputData, identifier)
-        runElement.execComplete.connect(self.operationDone)
-        runElement.removeSelf.connect(self.removeOperatorThread)
-
-        
-        if element["HighlightState"]:
-            self.updateStatus(element, True)
-        
-
-        runElement.start()
-
-        self.processHandles[identifier] = runElement
-        #logging.debug('Operator::createProcHandle() called - identifier: {:04d}'.format(identifier))
-
-        
-
-    def stopExec(self, id):
-        logging.debug('Operator::stopExec() called - id: 0x{:08x}'.format(id))
-        
-        for threadIdentifier, processHandle in self.processHandles.items():
-            if processHandle.element['Id'] == id:
-                processHandle.stop()
-
-    def stopAll(self):
-
-        logging.debug('Operator::stopAll() called')
-        logging.info('User command: Stop All')
-        for threadIdentifier, processHandle in self.processHandles.items():
-            processHandle.stop()
-
-    def startAll(self, config):
-
-        logging.debug('Operator::startAll() called')
-        logging.info('User command: Start All')
-        self.currentConfig = config
-        
-        startElements = [x for x in config if not x['Socket']]
+        logging.debug('OperatorStartAll::startAll() called')
+            
+        startElements = [x for x in self.operator.currentConfig if not x['Socket']]
 
         for startElement in startElements:
-
-            processes = filter(lambda item: item[1].element['Id'] == startElement['Id'], self.processHandles.items())
+            
+            
+            processes = filter(lambda item: item[1].element['Id'] == startElement['Id'], self.operator.processHandles.items())
             runningProcess = next(processes, None)
 
             if(runningProcess):
@@ -234,7 +166,244 @@ class Operator(QThread):
             else:
                 logging.debug('Operator::startAll() -Element started - {} - id: 0x{:08x}'.format(
                    startElement['ObjectName'], startElement['Id']))
-                self.createProcHandle(startElement)
+                
+                self.operator.createProcHandle(startElement)
+
+class OperatorElementOpDone(QRunnable):
+
+    def __init__(self, config, id, record, identifier, operator):
+        super(OperatorElementOpDone, self).__init__()
+
+
+        self.currentConfig  = config
+        self.id             = id
+        self.record         = record
+        self.identifier     = identifier
+        self.operator       = operator
+
+
+    def run(self):
+
+        logging.debug("OperatorElementOpDone::run() called")
+
+        cfgElement = [x for x in self.currentConfig if x['Id'] == self.id][0]
+
+        if cfgElement['Config']['GeneralConfig']['Logging'] and self.record.message:
+            logging.info('{} - {}'.format(cfgElement['ObjectName'], self.record.message))
+
+            data = {
+                'Id'        : cfgElement['Id'],
+                'ObjectName': cfgElement['ObjectName'],
+                'Message'   : self.record.message
+            }
+            address = {
+                'target'    : 'MainWindow',                        
+            }
+            cmd = { 
+                'cmd'       : 'ElementMessage',
+                'address'   : address,
+                'data'      : data
+            }
+
+            self.operator.emitCommand(cmd)
+
+
+        if cfgElement['Config']['GeneralConfig']['Debug'] :
+
+            data = {
+                'Id'        : cfgElement['Id'],
+                'AreaNo'    : cfgElement['AreaNo'],
+                'ObjectName': cfgElement['ObjectName'],
+                'Output'    : str(self.record.data) 
+            }
+            address = {
+                'target'    : 'MainWindow',                        
+            }
+            cmd = { 
+                'cmd'       : 'DebugOutput',
+                'address'   : address,
+                'data'      : data
+            }
+
+            self.operator.emitCommand(cmd)
+
+        # return if the element has no childs
+        if not cfgElement['Childs']:
+            return
+        
+        for childId in cfgElement['Childs']:
+            childElement = [x for x in self.currentConfig if x['Id'] == childId][0]
+            self.operator.createProcHandle(childElement, self.record.data)
+            self.operator.highlightConnection(self.id, childId, childElement['AreaNo'])
+
+class OperatorCreateProcHandle(QRunnable):
+    
+    def __init__(self, element, inputData, procHandles, operator):
+        super(OperatorCreateProcHandle, self).__init__()
+        self.element        = element
+        self.inputData      = inputData
+        self.procHandles    = procHandles
+        self.operator       = operator
+
+    def run(self):
+
+        # check if element is already running
+        if self.element['AllowStream']:
+            
+            runningInstance =  list(filter(lambda item: item[1].element['Id'] == self.element['Id'], self.procHandles.items())) 
+            # when runningInstance contains an element forward input data to it
+            if runningInstance:
+                # [0] first element in the list, [1] ProcessHandler
+                tagetProc = runningInstance[0][1]
+                tagetProc.feed(self.inputData)
+                return
+
+
+        identifier = self.operator.getIdent()
+        runElement = ProcessHandler(self.element, self.inputData, identifier, self.operator)
+
+        if self.element["HighlightState"]: 
+            self.operator.updateStatus(self.element, True)
+        
+        self.operator.addHandle(identifier, runElement)
+        self.operator.threadpool.start(runElement)
+        
+        logging.debug('Operator::createProcHandle() called - identifier: {:04d}'.format(identifier))
+
+class OperatorReturnElementState(QRunnable):
+
+    def __init__(self, processHandles, operator):
+        super(OperatorReturnElementState, self).__init__()
+        self.processHandles = processHandles
+        self.operator       = operator
+
+    def run(self):
+
+        for threadIdentifier, processHandle in self.processHandles.items():
+            #logging.debug('running elements: 0x{:08x}'.format(processHandle.element['Id']))
+            self.operator.updateStatus(processHandle.element, True)
+
+class OperatorStopExec(QRunnable):
+
+    def __init__(self, processHandles, id, operator):
+        super(OperatorStopExec, self).__init__()
+        self.processHandles = processHandles
+        self.operator       = operator
+        self.id             = id
+
+    def run(self):
+
+        for threadIdentifier, processHandle in self.processHandles.items():
+            if processHandle.element['Id'] == self.id:
+                processHandle.stop()
+
+class Operator(QObject):
+
+    currentConfig   = None
+    processHandles  = {}
+    command         = Signal(object) # command
+
+    def __init__(self):
+        super(Operator, self).__init__()
+
+
+        logging.debug('Operator::__init__() called')
+        self.threadpool         = QThreadPool.globalInstance()
+        self.procHandleMutex    = QMutex()
+
+        self._startAll          = OperatorStartAll(self)
+
+        self.identGenMutex      = QMutex()
+        self.n_ident            = 0
+
+
+    def start(self, config):
+
+        logging.info('<#>DAEMON STARTED<#>')
+
+        if not config: # return here when there is no config file
+            return
+
+        self.currentConfig = config
+
+        # check for autostart elements
+
+        startElements = [x for x in self.currentConfig if not x['Socket'] and x['Config']['GeneralConfig']['Autostart']]
+
+        for startElement in startElements:
+            logging.info("Autostart " + startElement['ObjectName'])
+            self.createProcHandle(startElement)
+
+    def getIdent(self):
+
+        self.identGenMutex.lock()
+        
+        self.n_ident += 1 
+        # check if current ident is already in use
+        while list(filter(lambda item: item[0] == self.n_ident, self.processHandles.items())):
+            self.n_ident += 1
+        
+
+        if not self.n_ident & 0x7fff:
+            self.n_ident = 0
+        self.identGenMutex.unlock()
+
+        return self.n_ident
+
+    def startExec(self, id, config):
+        logging.debug('Operator::startExec() called - id: 0x{:08x}'.format(id))
+        ## create processor and forward config and start filename
+
+        self.currentConfig = config
+        # https://stackoverflow.com/questions/34609935/passing-a-function-with-two-arguments-to-filter-in-python
+
+        # return first element which matches the ID
+        startElement = [x for x in config if x['Id'] == id][0]
+
+        self.createProcHandle(startElement)
+
+    def createProcHandle(self, element, inputData=None):    
+        #logging.debug('Operator::startExec() called - id: 0x{:08x}'.format(id))
+
+        self.procHandleMutex.lock()
+        processHandles = self.processHandles.copy()
+        self.procHandleMutex.unlock()
+
+        procHandle = OperatorCreateProcHandle(element, inputData, processHandles, self)
+
+        self.threadpool.start(procHandle)    
+
+    def addHandle(self, identifier, handle):
+        
+        self.procHandleMutex.lock()
+        self.processHandles[identifier] = handle
+        self.procHandleMutex.unlock()
+
+    def stopExec(self, id):
+        logging.debug('Operator::stopExec() called - id: 0x{:08x}'.format(id))
+
+        self.procHandleMutex.lock()
+        processHandles = self.processHandles.copy()
+        self.procHandleMutex.unlock()
+
+        stopOperator = OperatorStopExec(processHandles, id, self)
+
+        self.threadpool.start(stopOperator)
+
+    def stopAll(self):
+
+        logging.debug('Operator::stopAll() called')
+        logging.info('User command: Stop All')
+        self.procHandleMutex.lock()
+        for threadIdentifier, processHandle in self.processHandles.items():
+            processHandle.stop()
+        self.procHandleMutex.unlock()
+
+    def startAll(self, config):
+
+        logging.info('User command: Start All')
+        self.currentConfig = config
+        self.threadpool.start(self._startAll)
 
     def killAll(self):
         
@@ -242,32 +411,33 @@ class Operator(QThread):
         logging.info('User command: Kill All Processes')   
         # Separate dict must be created because call to removerOperatorThreads
         # modifies self.processHandles
-
+        self.procHandleMutex.lock()
         processes = dict(filter(lambda item: item[1].pid, self.processHandles.items())) 
-        
+        self.procHandleMutex.unlock()
         for threadIdentifier, processHandle in processes.items():
             os.kill(processHandle.pid, signal.SIGTERM)
             # removeOperatorThread is called in run() function of ProcessHandler
         
-
     def getElementStates(self):
         
         logging.debug('Operator::getElementStates() called')
+        
+        self.procHandleMutex.lock()
+        processHandles = self.processHandles.copy()
+        self.procHandleMutex.unlock()
 
-        for threadIdentifier, processHandle in self.processHandles.items():
-            #logging.debug('running elements: 0x{:08x}'.format(processHandle.element['Id']))
-            self.updateStatus(processHandle.element, True)
-
-        #if procHandle.element["HighlightState"]:
-        #    self.updateStatus(procHandle.element, False)
+        stateOperator = OperatorReturnElementState(processHandles, self)
 
+        self.threadpool.start(stateOperator)
+        
     def updateStatus(self, element, status):
         #start highlight
         # area
         # id
         # target = "Element"
         # cmd = UpdateElementStatus
-        logging.debug('Operator::updateStatus() called - {} - id: 0x{:08x}'.format(status, element['Id']))
+        #logging.debug('Operator::updateStatus() called - {} - id: 0x{:08x}'.format(status, element['Id']))
+
         address = {
             'target'    : 'Element',
             'area'      : element['AreaNo'],
@@ -284,7 +454,7 @@ class Operator(QThread):
 
     def highlightConnection(self, parentId, childId, wrkArea):
 
-        #logging.debug('Operator::updateStatus() called - {} - id: 0x{:08x}'.format(status, element['Id']))
+        logging.debug('Operator::updateStatus() called')
         address = {
             'target'    : 'WorkingArea',
             'area'      : wrkArea           
@@ -303,12 +473,16 @@ class Operator(QThread):
 
         self.command.emit(cmd)
 
+    def emitCommand(self, command):
+
+        self.command.emit(command)
+
     def operationDone(self, id, record, identifier):
 
-        #logging.debug('Operator::operationDone() result received - id: 0x{:08x}, ident: {:04d} data: {}'.format(id, identifier, record.data))
+        logging.debug('Operator::operationDone() result received - id: 0x{:08x}, ident: {:04d}'.format(id, identifier))
 
         if isinstance(record, GuiCMD):
-            #logging.info(record.text)
+
             address = {
                 'target'    : 'Element',  
                 'id'        : id                      
@@ -320,70 +494,20 @@ class Operator(QThread):
             }
             self.command.emit(cmd)
             return
-
-
-        cfgElement = [x for x in self.currentConfig if x['Id'] == id][0]
-
-        if cfgElement['Config']['GeneralConfig']['Logging'] and record.message:
-            logging.info('{} - {}'.format(cfgElement['ObjectName'], record.message))
-
-            data = {
-                'Id'        : cfgElement['Id'],
-                'ObjectName': cfgElement['ObjectName'],
-                'Message'   : record.message
-            }
-            address = {
-                'target'    : 'MainWindow',                        
-            }
-            cmd = { 
-                'cmd'       : 'ElementMessage',
-                'address'   : address,
-                'data'      : data
-            }
-            self.command.emit(cmd)
-
-
-        if cfgElement['Config']['GeneralConfig']['Debug'] :
-
-            data = {
-                'Id'        : cfgElement['Id'],
-                'AreaNo'    : cfgElement['AreaNo'],
-                'ObjectName': cfgElement['ObjectName'],
-                'Output'    : str(record.data) 
-            }
-            address = {
-                'target'    : 'MainWindow',                        
-            }
-            cmd = { 
-                'cmd'       : 'DebugOutput',
-                'address'   : address,
-                'data'      : data
-            }
-            self.command.emit(cmd)
-
-
-        # return if the element has no childs
-        if not cfgElement['Childs']:
-            return
         
-        for childId in cfgElement['Childs']:
-            childElement = [x for x in self.currentConfig if x['Id'] == childId][0]
-            self.createProcHandle(childElement)
-            self.highlightConnection(parentId=id, childId=childId, wrkArea=childElement['AreaNo'])
+        operationDoneRunnable = OperatorElementOpDone(self.currentConfig, id, record, identifier, self)
 
+        self.threadpool.start(operationDoneRunnable)
+        
     def removeOperatorThread(self, id, identifier):
-
-        logging.debug('Operator::removeOperatorThread() called - id: 0x{:08x}, ident: {:04d}'.format(id, identifier))
-        procHandle = self.processHandles[identifier]
         
+        logging.debug('Operator::removeOperatorThread() called - id: 0x{:08x}, ident: {:04d}'.format(id, identifier))
+
+        self.procHandleMutex.lock()
+        procHandle = self.processHandles[identifier]
         if procHandle.element["HighlightState"]:
             self.updateStatus(procHandle.element, False)
-        
-        procHandle.deleteLater()
+
         del self.processHandles[identifier]
 
-
-
-
-
-
+        self.procHandleMutex.unlock()
\ No newline at end of file
